c
c-----------------------------------------------------------------------
c
      logical function getopt (args, largs, nargs,
     &                         flgs, lflgs, nflgs,
     &                         opts, lopts, nopts, optarg)
c
c.....getopt - parses the command line for arguments, flags and options.
c     The routine returns .false. if an error occurs, .true. otherwise.
c
c     Argument is any token in the commmand line that doesn't begin with
c     '-'. A standalone '-' will be considered a special argument, used
c     to skip an arg without filling it.
c
c     Flag is any char of a token that starts with a '-'.
c
c     Option is a flag that accepts an argument, which must be the token
c     following the option.
c
c     A '--' in the input line signals the end of options and flags (in
c     this way, arguments beginning with '-' are allowed).
c
c..Input parameters:
c
c  nargs ......... maximum number of allowed arguments.
c  nflgs ......... number of valid flags.
c  flgs .......... list of valid flags.
c  nopts ......... number of valid options.
c  opts .......... list of valid options.
c
c..Output parameters:
c
c  args() ........ returns the values of the specified arguments.
c  largs() ....... returns .true. on each specified argument (that is,
c                  '-' will increase the count of arguments but with
c                  a .false. in lflags).
c  lflgs() ....... returns a .true. in the position of the flags
c                  actually found in the command line.
c  lopts() ....... returns .true. in the position of used options.
c  optarg() ...... contains the arguments associated to the options
c                  actually found.
c
c..Example:
c
c  Given a program that expects for the next entrance:
c
c       myprog [input] [output] [-abcde] [-o arg1] [-s arg2]
c
c  where square braces [] contain optional arguments, the main program
c  may use the next call to getopt to parse the command line:
c
c       nargs = 2
c       nflgs = 5
c       flgs = 'abcde'
c       nopts = 2
c       opts = 'os'
c       if (.not. getopt (args, largs, nargs, flgs, lflgs, nflgs,
c      &            opts, lopts, nopts, optarg)) then
c          --- Something went wrong ---
c       endif
c
c PARAMETERS:
c
      integer           nargs, nflgs, nopts
      logical           largs(*), lflgs(*), lopts(*)
      character*(*)     args(*), optarg(*)
      character*(*)     flgs, opts
c
c LOCAL VARIABLES:
c
      integer           leng, iargc
      integer           iargs, iflgs, iopts
      integer           ntokens, itoken, ltoken, lp
      character*(128)   token
      logical           endopts
c
c.....get the number of tokens in the command line:
c
      iargs = 0
      ntokens = iargc ()
      if (ntokens.lt.0) then
        getopt = .false.
        return
      endif
c
c.....loop over tokens:
c
      endopts = .false.
      itoken = 1
      do while (itoken.le.ntokens)
        call getarg (itoken, token)
        itoken = itoken + 1
        ltoken = leng (token)
c
c.....is it an arg? (or we found the end of options)
c
        if (token(1:1).ne.'-' .or. endopts) then
          if (iargs.lt.nargs) then
            iargs = iargs + 1
            args(iargs) = token(1:ltoken)
            largs(iargs) = .true.
          else
            write (0,*) 'getopt: too many arguments'
            getopt = .false.
            return
          endif
c
c.....dummy argument:
c
        else if (ltoken.eq.1) then
          if (iargs.lt.nargs) then
            iargs = iargs + 1
            args(iargs) = token(1:ltoken)
            largs(iargs) = .false.
          else
            write (0,*) 'getopt: too many arguments'
            getopt = .false.
            return
          endif
c
c.....end of options:
c
        else if (token(1:ltoken).eq.'--') then
          endopts = .true.
c
c.....everything else are flags and options:
c
        else
c
c.....loop for flags:
c
          do lp = 2, ltoken-1
            iflgs = 1
            do while (iflgs.lt.nflgs .and.
     &                token(lp:lp).ne.flgs(iflgs:iflgs))
              iflgs = iflgs + 1
            enddo
            if (iflgs.eq.nflgs .and.
     &          token(lp:lp).ne.flgs(iflgs:iflgs)) then
              write (0,*)
     &          'getopt: -'//token(lp:lp)//' is not a valid flag'
              getopt = .false.
              return
            endif
            lflgs(iflgs) = .true.
          enddo
c
c.....last one can be a flag or an option:
c
          lp = ltoken
          iflgs = 1
          do while (iflgs.lt.nflgs .and.
     &              token(lp:lp).ne.flgs(iflgs:iflgs))
            iflgs = iflgs + 1
          enddo
c
c.....it is a flag:
c
          if (iflgs.ne.nflgs .or.
     &        token(lp:lp).eq.flgs(iflgs:iflgs)) then
            lflgs(iflgs) = .true.
c
c.....not a flag:
c
          else
            iopts = 1
            do while (iopts.lt.nopts .and.
     &                token(lp:lp).ne.opts(iopts:iopts))
              iopts = iopts + 1
            enddo
c
c.....it is an option:
c
            if (iopts.ne.nopts .or.
     &          token(lp:lp).eq.opts(iopts:iopts)) then
              if (itoken.le.ntokens) then
                lopts(iopts) = .true.
                call getarg (itoken, token)
                itoken = itoken + 1
                ltoken = leng (token)
                optarg(iopts) = token(1:ltoken)
              else
                write (0,*)
     &             'getopt: missing argument in option -'//token(lp:lp)
                getopt = .false.
                return
              endif
c
c.....not an option:
c
            else
              write (0,*)
     &          'getopt: -'//token(lp:lp)//' is not a valid flag'
              getopt = .false.
              return
            endif
          endif
        endif
      enddo
c
c.....end of parsing ok:
c
      getopt = .true.
      return
      end
c
c ----------------------------------------------------------------------
c
*mdc*if lahey vms
*      integer function iargc ()
*c
*c.....iargc: substitute function for unix's library one. Returns the
*c     number of tokens in the command line; tokens are separated by
*c     blanks; in addition, everything between double quotes is
*c     considered in the same token
*c
*      character*(1)     blank,null,dquote
*      parameter ( blank=char(32) , null=char(0) , dquote=char(34) )
*      integer           leng, length, lp, argc
*      logical           done
*      parameter ( margs = 16 )
*      character*(128)   line, token(0:margs)
*      common /commline/ length, argc, line, token
*c
*c.....get command line
*c
*mdc*endif
*mdc*if lahey
*      call getcl(line)
*      length=leng(line)
*mdc*elsif vms
*      include           '($clidef)'
*      call cli$get_value ('$line',line,length)
*mdc*endif
*mdc*if lahey vms
*c
*c.....skip initial spaces
*c
*      argc = 0
*      lp = 1
*      done = .false.
*      do while (.not.done .and. lp.le.length)
*        if (line(lp:lp).ne.blank) then
*          done = .true.
*        else
*          lp = lp + 1
*        endif
*      enddo
*      if (.not.done) then
*        call error ('iargc', 'empty command line', warning)
*        iargc = -1
*        return
*      endif
*c
*c.....skip command name
*c
*      done = .false.
*      i = lp
*      do while (.not.done .and. lp.le.length)
*        if (line(lp:lp).eq.blank) then
*          done = .true.
*        else
*          lp = lp + 1
*        endif
*      enddo
*      token(argc) = line(i:lp-1)
*c
*c.....no arguments
*c
*      if (.not.done) then
*        iargc = 0
*        return
*      endif
*c
*c.....loop over tokens
*c
*      do while (lp.le.length)
*c
*c.....skip blanks
*c
*        done = .false.
*        do while (.not.done .and. lp.le.length)
*          if (line(lp:lp).ne.blank) then
*            done = .true.
*          else
*            lp = lp + 1
*          endif
*        enddo
*c
*c.....end of command line
*c
*        if (.not.done) then
*          iargc = argc
*          return
*        endif
*c
*c.....get next token
*c
*        argc = argc + 1
*        if (argc.gt.margs)
*     &    call error ('iargc', 'too many tokens',faterr)
*        token(argc) = null
*        i = 1
*  10    continue
*        done = .false.
*        do while (.not.done .and. lp.le.length)
*          if (line(lp:lp).eq.blank .or. line(lp:lp).eq.dquote) then
*            done = .true.
*          else
*            token(argc)(i:i) = line(lp:lp)
*            i = i + 1
*            lp = lp + 1
*          endif
*        enddo
*c
*c.....end of command line
*c
*        if (.not.done) then
*          iargc = argc
*          return
*        endif
*c
*c.....token in double quotes; the goto takes into account abc" d "efg
*c     as one token
*c
*        if (line(lp:lp).eq.dquote) then
*          if (lp.eq.length) then
*            write (0,*) 'iargc: unmatched '//dquote
*            iargc = -1
*            return
*          endif
*          lp = lp + 1
*          done = .false.
*          do while (.not.done .and. lp.le.length)
*            if (line(lp:lp).eq.dquote) then
*              done = .true.
*            else
*              token(argc)(i:i) = line(lp:lp)
*              i = i + 1
*              lp = lp + 1
*            endif
*          enddo
*          lp = lp + 1
*          if (.not.done) then
*            write (0,*) 'iargc: unmatched '//dquote
*            iargc = -1
*            return
*          endif
*          goto 10
*        endif
*      enddo
*c
*c.....return the number of arguments
*c
*      iargc = argc
*      return
*      end
*mdc*endif
c
c ----------------------------------------------------------------------
c
*mdc*if lahey vms
*      subroutine getarg (itoken, rtoken)
*c
*c.....getarg: substitute function for unix's library one. Returns the
*c     token number itoken in the command line; tokens are separated by
*c     blanks; in addition, everything between double quotes is
*c     considered to be in the same token, after removing the quotes
*c
*      integer           itoken
*      character*(*)     rtoken
*c
*      integer           length, argc
*      character*(1)     null
*      parameter ( null = char(0) )
*      parameter ( margs = 16 )
*      character*(128)   line, token(0:margs)
*      common /commline/ length, argc, line, token
*c
*c.....test for valid itoken
*c
*      if (itoken.lt.0 .or. itoken.gt.argc) then
*        write (0,*) 'getarg: invalid token number ',itoken
*        rtoken = null
*      else
*        rtoken = token(itoken)
*      endif
*      return
*      end
*mdc*endif
